Attribute VB_Name = "modIFileOperationProgressSink"
' Autor: F. Schler (frank@activevb.de)
' Datum: 12/2018

Option Explicit

' ----==== Const ====----
Private Const S_OK As Long = &H0
Private Const E_NOINTERFACE As Long = &H80004002
Private Const IID_IFileOperationProgressSink As String = "{04b0f1a7-9490-44bc-96e1-4296a31252e2}"

' ----==== Types ====----
' IFileOperationProgressSink
Private Type tIFileOperationProgressSink
    pVTable As Long
End Type

' IFileOperationProgressSink VTable
Private Type tIFileOperationProgressSinkVTable
    VTable(0 To 18) As Long
End Type

' ----==== Kernel32 API-Deklarationen ====----
Private Declare Sub CopyMemory Lib "kernel32" _
                    Alias "RtlMoveMemory" ( _
                    ByRef Destination As Any, _
                    ByRef Source As Any, _
                    ByVal Length As Long)

' ----==== Variablen ====----
Private m_IPtr As Long      ' Interface Pointer
Private m_RefCount As Long  ' Referenzzhler

' Klasse fr die Events
Private m_pInterface As Long
Private m_Interface As Object

' IFileOperationProgressSink
Private m_IFileOperationProgressSink As tIFileOperationProgressSink
Private m_IFileOperationProgressSinkVTable As tIFileOperationProgressSinkVTable

' Initialisiert das Interfcace IFileOperationProgressSink
Public Function IFileOperationProgressSink(ByVal cIFileOperationProgressSink As _
    Long) As Long

    ' Klasse, an die die Events weitergeleitet werden
    m_pInterface = cIFileOperationProgressSink

    ' ----==== Interface IFileOperationProgressSink  ====----
    With m_IFileOperationProgressSinkVTable
        
        ' VTable aufbauen
        .VTable(0) = ProcPtr(AddressOf QueryInterface)
        .VTable(1) = ProcPtr(AddressOf AddRef)
        .VTable(2) = ProcPtr(AddressOf Release)
        .VTable(3) = ProcPtr(AddressOf StartOperations)
        .VTable(4) = ProcPtr(AddressOf FinishOperations)
        .VTable(5) = ProcPtr(AddressOf PreRenameItem)
        .VTable(6) = ProcPtr(AddressOf PostRenameItem)
        .VTable(7) = ProcPtr(AddressOf PreMoveItem)
        .VTable(8) = ProcPtr(AddressOf PostMoveItem)
        .VTable(9) = ProcPtr(AddressOf PreCopyItem)
        .VTable(10) = ProcPtr(AddressOf PostCopyItem)
        .VTable(11) = ProcPtr(AddressOf PreDeleteItem)
        .VTable(12) = ProcPtr(AddressOf PostDeleteItem)
        .VTable(13) = ProcPtr(AddressOf PreNewItem)
        .VTable(14) = ProcPtr(AddressOf PostNewItem)
        .VTable(15) = ProcPtr(AddressOf UpdateProgress)
        .VTable(16) = ProcPtr(AddressOf ResetTimer)
        .VTable(17) = ProcPtr(AddressOf PauseTimer)
        .VTable(18) = ProcPtr(AddressOf ResumeTimer)
    End With

    With m_IFileOperationProgressSink

        ' Pointer auf die VTable
        .pVTable = VarPtr(m_IFileOperationProgressSinkVTable)

    End With

    ' Pointer auf das IFileOperationProgressSink
    m_IPtr = VarPtr(m_IFileOperationProgressSink)

    IFileOperationProgressSink = m_IPtr

End Function

Private Function ProcPtr(ByVal ptr As Long) As Long

    ProcPtr = ptr

End Function

' ----==== Interface IUnknown Func ====----
'
' fr alle Interfaces in diesem Modul gleich
Private Function QueryInterface(ByVal this As Long, ByRef riid As GUID, ByRef _
    pvObj As Long) As Long

    Dim lRet As Long
    
    Select Case UCase$(Guid2String(riid))

    ' Wenn nach dem Interface IFileOperationProgressSink gefragt wird.
    Case UCase$(IID_IFileOperationProgressSink)
        
        ' dann muss ein AddRef aufgerufen werden.
        Call AddRef(this)

        ' Pointer auf das Interface
        pvObj = VarPtr(m_IFileOperationProgressSink)
        
        ' OK zurck geben, Interface ist vorhanden
        lRet = S_OK

    Case Else
    
        ' alles andere ignorieren
        
        pvObj = 0&
        
        lRet = E_NOINTERFACE
    
    End Select

    QueryInterface = lRet

End Function

' fr alle Interfaces in diesem Modul gleich
Private Function AddRef(ByVal this As Long) As Long

    ' Referenzzhler addieren
    m_RefCount = m_RefCount + 1

    ' Referenzhler zurck liefern
    AddRef = m_RefCount

End Function

' fr alle Interfaces in diesem Modul gleich
Private Function Release(ByVal this As Long) As Long
    
    ' Referenzzhler subtrahieren
    m_RefCount = m_RefCount - 1

    If m_RefCount = 0 Then
    
        ' alles Aufrumen und Freigeben
        m_IFileOperationProgressSink.pVTable = 0&
        Erase m_IFileOperationProgressSinkVTable.VTable
        
        m_IPtr = 0&
        
        m_pInterface = 0&
    
    End If
    
    ' Referenzhler zurck liefern
    Release = m_RefCount

End Function

' ----==== Interface IFileOperationProgressSink Func ====----
Private Function StartOperations(ByVal this As Long) As Long

    If m_pInterface <> 0& Then

        ' Objekt vom Pointer
        Call CopyMemory(m_Interface, m_pInterface, 4)
            
        ' Funktion in der Klasse aufrufen
        Call m_Interface.StartOperations

        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)

    End If

End Function

Private Function FinishOperations(ByVal this As Long, ByVal hrResult As Long) As _
    Long
    
    If m_pInterface <> 0& Then

        ' Objekt vom Pointer
        Call CopyMemory(m_Interface, m_pInterface, 4)
            
        ' Funktion in der Klasse aufrufen
        Call m_Interface.FinishOperations(hrResult)

        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)

    End If

End Function

Private Function PreRenameItem(ByVal this As Long, ByVal dwFlags As _
    TRANSFER_SOURCE_FLAGS, ByVal psiItem As Long, ByVal pszNewName As Long) As _
    Long

    If m_pInterface <> 0& Then

        ' Objekt vom Pointer
        Call CopyMemory(m_Interface, m_pInterface, 4)
            
        ' Funktion in der Klasse aufrufen
        Call m_Interface.PreRenameItem(dwFlags, psiItem, pszNewName)

        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)

    End If
    
End Function

Private Function PostRenameItem(ByVal this As Long, ByVal dwFlags As _
    TRANSFER_SOURCE_FLAGS, ByVal psiItem As Long, ByVal pszNewName As Long, ByVal _
    hrRename As Long, ByVal psiNewlyCreated As Long) As Long
    
    If m_pInterface <> 0& Then

        ' Objekt vom Pointer
        Call CopyMemory(m_Interface, m_pInterface, 4)
            
        ' Funktion in der Klasse aufrufen
        Call m_Interface.PostRenameItem(dwFlags, psiItem, _
            pszNewName, hrRename, psiNewlyCreated)

        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)

    End If

End Function

Private Function PreMoveItem(ByVal this As Long, ByVal dwFlags As _
    TRANSFER_SOURCE_FLAGS, ByVal psiItem As Long, ByVal psiDestinationFolder As _
    Long, ByVal pszNewName As Long) As Long

    If m_pInterface <> 0& Then

        ' Objekt vom Pointer
        Call CopyMemory(m_Interface, m_pInterface, 4)
            
        ' Funktion in der Klasse aufrufen
        Call m_Interface.PreMoveItem(dwFlags, psiItem, _
            psiDestinationFolder, pszNewName)

        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)

    End If
    
End Function

Private Function PostMoveItem(ByVal this As Long, ByVal dwFlags As _
    TRANSFER_SOURCE_FLAGS, ByVal psiItem As Long, ByVal psiDestinationFolder As _
    Long, ByVal pszNewName As Long, ByVal hrMove As Long, ByVal psiNewlyCreated As _
    Long) As Long

    If m_pInterface <> 0& Then

        ' Objekt vom Pointer
        Call CopyMemory(m_Interface, m_pInterface, 4)
            
        ' Funktion in der Klasse aufrufen
        Call m_Interface.PostMoveItem(dwFlags, psiItem, _
            psiDestinationFolder, pszNewName, hrMove, psiNewlyCreated)

        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)

    End If
    
End Function

Private Function PreCopyItem(ByVal this As Long, ByVal dwFlags As _
    TRANSFER_SOURCE_FLAGS, ByVal psiItem As Long, ByVal psiDestinationFolder As _
    Long, ByVal pszNewName As Long) As Long

    If m_pInterface <> 0& Then

        ' Objekt vom Pointer
        Call CopyMemory(m_Interface, m_pInterface, 4)
            
        ' Funktion in der Klasse aufrufen
        Call m_Interface.PreCopyItem(dwFlags, psiItem, _
            psiDestinationFolder, pszNewName)

        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)

    End If
    
End Function

Private Function PostCopyItem(ByVal this As Long, ByVal dwFlags As _
    TRANSFER_SOURCE_FLAGS, ByVal psiItem As Long, ByVal psiDestinationFolder As _
    Long, ByVal pszNewName As Long, ByVal hrCopy As Long, ByVal psiNewlyCreated As _
    Long) As Long

    If m_pInterface <> 0& Then

        ' Objekt vom Pointer
        Call CopyMemory(m_Interface, m_pInterface, 4)
            
        ' Funktion in der Klasse aufrufen
        Call m_Interface.PostCopyItem(dwFlags, psiItem, _
            psiDestinationFolder, pszNewName, hrCopy, psiNewlyCreated)

        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)

    End If
    
End Function

Private Function PreDeleteItem(ByVal this As Long, ByVal dwFlags As _
    TRANSFER_SOURCE_FLAGS, ByVal psiItem As Long) As Long

    If m_pInterface <> 0& Then

        ' Objekt vom Pointer
        Call CopyMemory(m_Interface, m_pInterface, 4)
            
        ' Funktion in der Klasse aufrufen
        Call m_Interface.PreDeleteItem(dwFlags, psiItem)

        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)

    End If
    
End Function

Private Function PostDeleteItem(ByVal this As Long, ByVal dwFlags As _
    TRANSFER_SOURCE_FLAGS, ByVal psiItem As Long, ByVal hrDelete As Long, ByVal _
    psiNewlyCreated As Long) As Long

    If m_pInterface <> 0& Then

        ' Objekt vom Pointer
        Call CopyMemory(m_Interface, m_pInterface, 4)
            
        ' Funktion in der Klasse aufrufen
        Call m_Interface.PostDeleteItem(dwFlags, psiItem, hrDelete, _
            psiNewlyCreated)

        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)

    End If
    
End Function

Private Function PreNewItem(ByVal this As Long, ByVal dwFlags As _
    TRANSFER_SOURCE_FLAGS, ByVal psiDestinationFolder As Long, ByVal pszNewName As _
    Long) As Long

    If m_pInterface <> 0& Then

        ' Objekt vom Pointer
        Call CopyMemory(m_Interface, m_pInterface, 4)
            
        ' Funktion in der Klasse aufrufen
        Call m_Interface.PreNewItem(dwFlags, psiDestinationFolder, _
            pszNewName)

        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)

    End If
    
End Function

Private Function PostNewItem(ByVal this As Long, ByVal dwFlags As _
    TRANSFER_SOURCE_FLAGS, ByVal psiDestinationFolder As Long, ByVal pszNewName As _
    Long, ByVal pszTemplateName As Long, ByVal dwFileAttributes As Long, ByVal _
    hrNew As Long, ByVal psiNewItem As Long) As Long

    If m_pInterface <> 0& Then

        ' Objekt vom Pointer
        Call CopyMemory(m_Interface, m_pInterface, 4)
            
        ' Funktion in der Klasse aufrufen
        Call m_Interface.PostNewItem(dwFlags, psiDestinationFolder, _
            pszNewName, pszTemplateName, dwFileAttributes, hrNew, psiNewItem)

        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)

    End If
    
End Function

Private Function UpdateProgress(ByVal this As Long, ByVal iWorkTotal As Long, _
    ByVal iWorkSoFar As Long) As Long

    If m_pInterface <> 0& Then

        ' Objekt vom Pointer
        Call CopyMemory(m_Interface, m_pInterface, 4)
            
        ' Funktion in der Klasse aufrufen
        Call m_Interface.UpdateProgress(iWorkTotal, iWorkSoFar)

        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)

    End If
    
End Function

Private Function ResetTimer(ByVal this As Long) As Long

    If m_pInterface <> 0& Then

        ' Objekt vom Pointer
        Call CopyMemory(m_Interface, m_pInterface, 4)
            
        ' Funktion in der Klasse aufrufen
        Call m_Interface.ResetTimer

        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)

    End If
    
End Function

Private Function PauseTimer(ByVal this As Long) As Long

    If m_pInterface <> 0& Then

        ' Objekt vom Pointer
        Call CopyMemory(m_Interface, m_pInterface, 4)
            
        ' Funktion in der Klasse aufrufen
        Call m_Interface.PauseTimer

        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)

    End If
    
End Function

Private Function ResumeTimer(ByVal this As Long) As Long

    If m_pInterface <> 0& Then

        ' Objekt vom Pointer
        Call CopyMemory(m_Interface, m_pInterface, 4)
            
        ' Funktion in der Klasse aufrufen
        Call m_Interface.ResetTimer

        ' Objekt lschen
        Call CopyMemory(m_Interface, 0&, 4)

    End If
    
End Function

